/***************************************************************************
 * Artekit STM32 DIY lightsaber audio board
 * Copyright (C) 2013 Artekit Italy
 * Written by Ivan Meleca
 * http://www.artekit.eu
 
### config.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "config.h"
#include "uart.h"
#include "flash.h"

/* In this variable we store the read header */
SND_HEADER snd_header;

/* Array of sound descriptors */
SND_DESC snd_desc[SND_MAX_DESC];

/* This function checks for a character coming through the uart. Used to check if we have to 
 * download the a new waveforms file.
 */
uint8_t configPoll(void)
{
	uint8_t c;
		
	if (uartReadChar(&c))
		return 1;
	
	return 0;
}

/* Receives the new waveforms file */
void configReceive(void)
{
	uint8_t c;
	uint8_t i;
	uint32_t addr = 0;

	/* empty possible cr lf and garbage */
	for (i = 0; i < 0xFF; i++)
		uartReadChar(&c);
	
	uartWriteLine("Press any key to erase the flash and download the file.");
	
	while (!uartReadChar(&c));
	
	/* empty possible cr lf and garbage */
	for (i = 0; i < 0xFF; i++)
		uartReadChar(&c);
	
	/* Erase the entire serial flash */
	flashChipErase();
		
	uartWriteLine("Start sending the file. Reset the board when finished.");
	
	while (i)
	{
		/* Read from UART, write into the serial flash */
		if (uartReadChar(&c))
		{
			flashWrite(addr, &c, 1);
			addr++;
		}		
	}
}

/* Load the contents of the serial flash (header and descriptors) */
uint8_t configLoad(void)
{
	/* Read the header */
	flashRead(0x00, (uint8_t*) &snd_header, sizeof(snd_header));
	
	/* Minimum sanity check */
	if (snd_header.ver != 1) return 0;
	if (snd_header.cnt > SND_MAX_DESC) return 0;
	
	/* Read every descriptor */
	flashRead(sizeof(snd_header), (uint8_t*) &snd_desc, sizeof(SND_DESC) * snd_header.cnt);
	
	return 1;
}
